
********************************************
* File Locations                           *
********************************************

local excel_file "Levin et al. Supplementary Data 14aug2020.xlsx"

*********************************************
* Part I: Benchmark Metaregression Analysis *
*********************************************

* Import data from benchmark studies

clear 
graph drop _all
import excel using "`excel_file'", sheet("Non-US Seroprevalence Studies") firstrow case(lower)
gen usa_flag = 0
save "temp_store.dta", replace 

* Import data from other U.S. studies

clear
import excel using "`excel_file'", sheet("US Studies") firstrow case(lower)
destring infrate_ci95_low infrate_ci95_high ifr_ci95_low ifr_ci95_high, force replace
gen usa_flag = cond(inlist(substr(study,1,3),"Atl","Ind","New","Sal"),1,2)
append using "temp_store.dta"
save "temp_store.dta", replace 

* Import data from track & trace countries

clear
import excel using "`excel_file'", sheet("Comprehensive Tracing") firstrow case(lower)
destring infrate_ci95_low infrate_ci95_high ifr_ci95_low ifr_ci95_high, force replace
gen usa_flag = 0
append using "temp_store.dta"
capture drop l m n o p q r s
capture erase temp_store.dta

* Create indicator variables

gen study_agegroup = study + " " + agegroup
sort usa_flag study median_age
egen istudy = group(usa_flag study)
sort istudy
by istudy: gen study_id = 10*istudy + _n
quietly gen log_ifr_studyUS1 = log(ifr) if usa_flag==1
quietly gen log_ifr_studyUS2 = log(ifr) if usa_flag==2
quietly gen log_ifr_studyX = log(ifr) if usa_flag==0
separate log_ifr_studyUS1 if usa_flag==1, by(study) veryshortlabel 
separate log_ifr_studyUS2 if usa_flag==2, by(study) veryshortlabel 
separate log_ifr_studyX if usa_flag==0, by(study) veryshortlabel 
quietly gen ifr_studyUS1 = ifr if usa_flag==1
quietly gen ifr_studyUS2 = ifr if usa_flag==2
quietly gen ifr_studyX = ifr if usa_flag==0
separate ifr_studyUS1 if usa_flag==1, by(study) veryshortlabel 
separate ifr_studyUS2 if usa_flag==2, by(study) veryshortlabel 
separate ifr_studyX if usa_flag==0, by(study) veryshortlabel 

* Compute each standard error using the 95% confidence interval and Gaussian thresholds 
* i.e., 95% confidence interval = mean +/- 1.96*SE

gen ifr_se = (ifr_ci95_high-ifr_ci95_low)/3.84

* Generate log(ifr) and compute standard error using the delta method
* i.e., SE(log(xhat)) = SE(xhat)/xhat

gen log_ifr = log(ifr)
gen log_ifr_se = ifr_se/ifr
gen log_ifr_precision = 1/log_ifr_se^2

* Generate aggregate IFRs

sort istudy
quietly replace infections = 0.01*infectionrate*population if missing(infections)
quietly gen infections_ci95_low = 0.01*infrate_ci95_low*population 
quietly gen infections_ci95_high = 0.01*infrate_ci95_high*population 

by istudy: gen study_flag = cond(_n==1,1,0)
by istudy: egen total_population = sum(population)
by istudy: egen total_infections = sum(infections)
by istudy: egen total_infections_ci95_low = sum(infections_ci95_low) 
by istudy: egen total_infections_ci95_high = sum(infections_ci95_high) 
by istudy: egen total_deaths = sum(deaths)
by istudy: gen total_ifr = 100*total_deaths/total_infections 
by istudy: gen total_ifr_ci95_low = 100*total_deaths/total_infections_ci95_high 
by istudy: gen total_ifr_ci95_high = 100*total_deaths/total_infections_ci95_low 

list istudy study total_ifr total_ifr_ci95_low total_ifr_ci95_high if study_flag==1 & !missing(study)
drop study_flag

* Exclude observations with median age < 35 years, very low prevalence, or zero deaths

drop if median_age < 35
list study agegroup if infrate_ci95_low <= 0.1 & usa_flag>0
drop if infrate_ci95_low <= 0.1 & usa_flag>0
list study agegroup if deaths==0
drop if deaths==0

* List observations

sort study_id
list study_id study median_age log_ifr log_ifr_se if !missing(study)
sort median_age

* Run benchmark metaregression

meta set log_ifr log_ifr_se, studylabel(study_agegroup)
meta regress median_age, random se(kh)
local tau2 = e(tau2)
quietly gen metareg_wt = _meta_regweight

predict ueffect, reffects se(ueffect_se)
gen ueffect_norm = ueffect/ueffect_se
qnorm ueffect_norm, mlabel(istudy) name(meta_qnorm)

* Append data with sequential series of median_age 
* for generating smooth predicted curve in plots

local ncount = _N
set obs `=`ncount' + 121'
forvalues ix = 1/121 {
  quietly replace median_age = 34.5 + 0.5*`ix' if _n == `=`ncount' + `ix''
  }
sort median_age

* Use the predict function of metareg to generate fitted values,
* confidence intervals, and prediction intervals for log model

predict fit, xb
predict stdp, stdp
gen stdf = (`tau2' + stdp^2)^0.5

quietly gen conf_low = fit - 1.96*stdp 
quietly gen conf_high = fit + 1.96*stdp
quietly gen pred_low = fit - 1.96*stdf
quietly gen pred_high = fit + 1.96*stdf

* Compute estimates and intervals for level of IFR in percent

quietly gen expfit = exp(fit) 
quietly gen exp_conf_low = exp(conf_low) 
quietly gen exp_conf_high = min(exp(conf_high),100) 
quietly gen exp_pred_low = exp(pred_low) 
quietly gen exp_pred_high = min(exp(pred_high),100) 

quietly replace fit = . if !missing(study)
quietly replace conf_low = . if !missing(study)
quietly replace conf_high = . if !missing(study)
quietly replace pred_low = . if !missing(study)
quietly replace pred_high = . if !missing(study)
quietly replace expfit = . if !missing(study)
quietly replace exp_conf_low = . if !missing(study)
quietly replace exp_conf_high = . if !missing(study)
quietly replace exp_pred_low = . if !missing(study)
quietly replace exp_pred_high = . if !missing(study)

* Produce chart of meta-regression in logs

quietly gen exp_conf_hitop = min(exp_conf_high,25) if missing(study)
quietly gen exp_pred_hitop = min(exp_pred_high,25) if missing(study)

local bot_age = 35
local top_age = 90
twoway line fit median_age if inrange(median_age,`bot_age',`top_age'), ///
  	      legend(label(1 "Meta-Regression")) lcolor(red) lwidth(thick) ///
	|| rarea pred_low pred_high median_age if inrange(median_age,`bot_age',`top_age'), ///
          legend(label(2 "Prediction Interval")) fcolor(purple%10) lcolor(purple) ///
	|| rarea conf_low conf_high median_age if inrange(median_age,`bot_age',`top_age'), ///
          legend(label(3 "Confidence Interval")) fcolor(purple%30) lcolor(blue) ///
    || scatter log_ifr_studyX?* median_age if inrange(median_age,`bot_age',`top_age'), ///
         msymbol(O S T D o s t d Oh Sh Th Dh) ///
	     mcolor(blue magenta navy ltblue lavender maroon cranberry orange dkorange cyan dkgreen purple) /// 
    || scatter log_ifr_studyUS1?* median_age if inrange(median_age,`bot_age',`top_age'), ///
         msymbol(oh sh dh th) ///
	     mcolor(green emerald mint forest_green) ///
    || scatter log_ifr_studyUS2?* median_age if inrange(median_age,`bot_age',`top_age'), ///
         msymbol(X X X X X X X X) msize(medlarge medlarge medlarge medlarge medlarge medlarge medlarge medlarge) ///
	     mcolor(black teal brown gray sienna ebblue khaki sand) ///   	   
		 legend(pos(11) ring(0) col(4) colfirst size(small) ///
  	     order(1 3 2 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27) ///
              holes(16 23 24 31 32) region(lwidth(none) fcolor(none))) ///
	   xlabel(30 40 50 60 70 80 90) ylabel(-5 -2.5 0 2.5 5 7.5 10) xtitle("Median Age",height(5)) ///
	   name(ifr_meta_log_plot) title("Logarithm of IFR", placement(ne) size(medsmall))

* Produce chart of meta-regression in levels

local bot_age = 45
local top_age = 85
twoway line expfit median_age if inrange(median_age,`bot_age',`top_age'), ///
  	   legend(label(1 "Meta-Regression")) lcolor(red) lwidth(thick) xlabel(45 55 65 75 85) ///
	|| rarea exp_pred_low exp_pred_hitop median_age if inrange(median_age,`bot_age',`top_age'), ///
       legend(label(2 "Prediction Interval")) fcolor(purple%10) lcolor(purple) ///
	|| rarea exp_conf_low exp_conf_hitop median_age if inrange(median_age,`bot_age',`top_age'), ///
       legend(label(3 "Confidence Interval")) fcolor(purple%30) lcolor(blue) ///
    || scatter ifr_studyX?* median_age if inrange(median_age,`bot_age',`top_age'), ///
         msymbol(O S T D o s t d D S T O) ///
	     mcolor(blue sienna magenta navy lavender maroon cranberry orange dkorange cyan gray purple) /// 
    || scatter ifr_studyUS1?* median_age if inrange(median_age,`bot_age',`top_age'), ///
         msymbol(oh sh dh th) ///
	     mcolor(green emerald mint forest_green) ///
    || scatter ifr_studyUS2?* median_age if inrange(median_age,`bot_age',`top_age'), ///
         msymbol(X X X X X X X X) msize(medlarge medlarge medlarge medlarge medlarge medlarge medlarge medlarge) ///
	     mcolor(black teal brown gray sienna ebblue khaki sand) ///   	   
   	   legend(pos(11) ring(0) region(lwidth(none) fcolor(none)) col(2) colfirst size(small) ///
	   order(1 3 2 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27) ///
	   holes(28 29 30)) ///
	   xlabel(45 55 65 75 85) ylabel(0 5 10 15 20 25) ///
	   xtitle("Median Age",height(5)) title("Percent", placement(ne) size(small)) ///
	   name(ifr_meta_plot)

	   
